@@ -1,26 +1,22 @@
-# test_config.py
 # Copyright (C) 2008, 2009 Michael Trier (mtrier@gmail.com) and contributors
 #
-# This module is part of GitPython and is released under
-# the BSD License: https://opensource.org/license/bsd-3-clause/
+# This module is part of GitPython and is released under the
+# 3-Clause BSD License: https://opensource.org/license/bsd-3-clause/
 
 import glob
 import io
 import os
+import os.path as osp
+import sys
 from unittest import mock
 
+import pytest
+
 from git import GitConfigParser
 from git.config import _OMD, cp
-from test.lib import (
-    TestCase,
-    fixture_path,
-    SkipTest,
-)
-from test.lib import with_rw_directory
-
-import os.path as osp
 from git.util import rmfile
 
+from test.lib import SkipTest, TestCase, fixture_path, with_rw_directory
 
 _tc_lock_fpaths = osp.join(osp.dirname(__file__), "fixtures/*.lock")
 
@@ -46,29 +42,30 @@ class TestBase(TestCase):
         return sio
 
     def test_read_write(self):
-        # writer must create the exact same file as the one read before
+        # The writer must create the exact same file as the one read before.
         for filename in ("git_config", "git_config_global"):
             file_obj = self._to_memcache(fixture_path(filename))
             with GitConfigParser(file_obj, read_only=False) as w_config:
-                w_config.read()  # enforce reading
+                w_config.read()  # Enforce reading.
                 assert w_config._sections
-                w_config.write()  # enforce writing
+                w_config.write()  # Enforce writing.
 
-                # we stripped lines when reading, so the results differ
+                # We stripped lines when reading, so the results differ.
                 assert file_obj.getvalue()
                 self.assertEqual(
                     file_obj.getvalue(),
                     self._to_memcache(fixture_path(filename)).getvalue(),
                 )
 
-                # creating an additional config writer must fail due to exclusive access
+                # Creating an additional config writer must fail due to exclusive
+                # access.
                 with self.assertRaises(IOError):
                     GitConfigParser(file_obj, read_only=False)
 
-                # should still have a lock and be able to make changes
+                # Should still have a lock and be able to make changes.
                 assert w_config._lock._has_lock()
 
-                # changes should be written right away
+                # Changes should be written right away.
                 sname = "my_section"
                 oname = "mykey"
                 val = "myvalue"
@@ -93,13 +90,14 @@ class TestBase(TestCase):
 
     def test_includes_order(self):
         with GitConfigParser(list(map(fixture_path, ("git_config", "git_config_global")))) as r_config:
-            r_config.read()  # enforce reading
-            # Simple inclusions, again checking them taking precedence
+            r_config.read()  # Enforce reading.
+            # Simple inclusions, again checking them taking precedence.
             assert r_config.get_value("sec", "var0") == "value0_included"
-            # This one should take the git_config_global value since included
-            # values must be considered as soon as they get them
+            # This one should take the git_config_global value since included values
+            # must be considered as soon as they get them.
             assert r_config.get_value("diff", "tool") == "meld"
             try:
+                # FIXME: Split this assertion out somehow and mark it xfail (or fix it).
                 assert r_config.get_value("sec", "var1") == "value1_main"
             except AssertionError as e:
                 raise SkipTest("Known failure -- included values are not in effect right away") from e
@@ -110,16 +108,17 @@ class TestBase(TestCase):
         gcp = GitConfigParser(fpl, read_only=False)
         with gcp as cw:
             cw.set_value("include", "some_value", "a")
-        # entering again locks the file again...
+        # Entering again locks the file again...
         with gcp as cw:
             cw.set_value("include", "some_other_value", "b")
-            # ...so creating an additional config writer must fail due to exclusive access
+            # ...so creating an additional config writer must fail due to exclusive
+            # access.
             with self.assertRaises(IOError):
                 GitConfigParser(fpl, read_only=False)
         # but work when the lock is removed
         with GitConfigParser(fpl, read_only=False):
             assert osp.exists(fpl)
-            # reentering with an existing lock must fail due to exclusive access
+            # Reentering with an existing lock must fail due to exclusive access.
             with self.assertRaises(IOError):
                 gcp.__enter__()
 
@@ -129,7 +128,7 @@ class TestBase(TestCase):
             ev = "ruby -e '\n"
             ev += "		system %(git), %(merge-file), %(--marker-size=%L), %(%A), %(%O), %(%B)\n"
             ev += "		b = File.read(%(%A))\n"
-            ev += "		b.sub!(/^<+ .*\\nActiveRecord::Schema\\.define.:version => (\\d+). do\\n=+\\nActiveRecord::Schema\\."  # noqa E501
+            ev += "		b.sub!(/^<+ .*\\nActiveRecord::Schema\\.define.:version => (\\d+). do\\n=+\\nActiveRecord::Schema\\."  # noqa: E501
             ev += "define.:version => (\\d+). do\\n>+ .*/) do\n"
             ev += "		  %(ActiveRecord::Schema.define(:version => #{[$1, $2].max}) do)\n"
             ev += "		end\n"
@@ -143,6 +142,14 @@ class TestBase(TestCase):
             )
             self.assertEqual(len(config.sections()), 23)
 
+    def test_config_value_with_trailing_new_line(self):
+        config_content = b'[section-header]\nkey:"value\n"'
+        config_file = io.BytesIO(config_content)
+        config_file.name = "multiline_value.config"
+
+        git_config = GitConfigParser(config_file)
+        git_config.read()  # This should not throw an exception
+
     def test_base(self):
         path_repo = fixture_path("git_config")
         path_global = fixture_path("git_config_global")
@@ -151,7 +158,7 @@ class TestBase(TestCase):
         num_sections = 0
         num_options = 0
 
-        # test reader methods
+        # Test reader methods.
         assert r_config._is_initialized is False
         for section in r_config.sections():
             num_sections += 1
@@ -164,7 +171,7 @@ class TestBase(TestCase):
                 assert "\n" not in option
                 assert "\n" not in val
 
-                # writing must fail
+                # Writing must fail.
                 with self.assertRaises(IOError):
                     r_config.set(section, option, None)
                 with self.assertRaises(IOError):
@@ -176,11 +183,11 @@ class TestBase(TestCase):
         assert num_sections and num_options
         assert r_config._is_initialized is True
 
-        # get value which doesn't exist, with default
+        # Get value which doesn't exist, with default.
         default = "my default value"
         assert r_config.get_value("doesnt", "exist", default) == default
 
-        # it raises if there is no default though
+        # It raises if there is no default though.
         with self.assertRaises(cp.NoSectionError):
             r_config.get_value("doesnt", "exist")
 
@@ -189,13 +196,9 @@ class TestBase(TestCase):
         def write_test_value(cw, value):
             cw.set_value(value, "value", value)
 
-        # end
-
         def check_test_value(cr, value):
             assert cr.get_value(value, "value") == value
 
-        # end
-
         # PREPARE CONFIG FILE A
         fpa = osp.join(rw_dir, "a")
         with GitConfigParser(fpa, read_only=False) as cw:
@@ -224,10 +227,10 @@ class TestBase(TestCase):
         with GitConfigParser(fpa, read_only=True) as cr:
             for tv in ("a", "b", "c"):
                 check_test_value(cr, tv)
-            # end for each test to verify
+            # END for each test to verify
             assert len(cr.items("include")) == 8, "Expected all include sections to be merged"
 
-        # test writable config writers - assure write-back doesn't involve includes
+        # Test writable config writers - assure write-back doesn't involve includes.
         with GitConfigParser(fpa, read_only=False, merge_includes=True) as cw:
             tv = "x"
             write_test_value(cw, tv)
@@ -236,20 +239,25 @@ class TestBase(TestCase):
             with self.assertRaises(cp.NoSectionError):
                 check_test_value(cr, tv)
 
-        # But can make it skip includes altogether, and thus allow write-backs
+        # But can make it skip includes altogether, and thus allow write-backs.
         with GitConfigParser(fpa, read_only=False, merge_includes=False) as cw:
             write_test_value(cw, tv)
 
         with GitConfigParser(fpa, read_only=True) as cr:
             check_test_value(cr, tv)
 
+    @pytest.mark.xfail(
+        sys.platform == "win32",
+        reason='Second config._has_includes() assertion fails (for "config is included if path is matching git_dir")',
+        raises=AssertionError,
+    )
     @with_rw_directory
     def test_conditional_includes_from_git_dir(self, rw_dir):
-        # Initiate repository path
+        # Initiate repository path.
         git_dir = osp.join(rw_dir, "target1", "repo1")
         os.makedirs(git_dir)
 
-        # Initiate mocked repository
+        # Initiate mocked repository.
         repo = mock.Mock(git_dir=git_dir)
 
         # Initiate config files.
@@ -312,11 +320,11 @@ class TestBase(TestCase):
 
     @with_rw_directory
     def test_conditional_includes_from_branch_name(self, rw_dir):
-        # Initiate mocked branch
+        # Initiate mocked branch.
         branch = mock.Mock()
         type(branch).name = mock.PropertyMock(return_value="/foo/branch")
 
-        # Initiate mocked repository
+        # Initiate mocked repository.
         repo = mock.Mock(active_branch=branch)
 
         # Initiate config files.
